(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('d3-selection')) :
	typeof define === 'function' && define.amd ? define(['exports', 'd3-selection'], factory) :
	(factory((global.d3 = global.d3 || {}),global.d3));
}(this, (function (exports,d3Selection) { 'use strict';

var setInstance = function setInstance(node, value) {
  node.__instance__ = value;
}; // no operation
var getInstance = function getInstance(node) {
  return node.__instance__;
};
var noop = function noop() {};

var component = function (tagName, className) {
  var create = noop;
  var render = noop;
  var _destroy = noop;
  var key = void 0;

  function component(selection, data, context) {
    var instances = (selection.nodeName ? d3Selection.select(selection) : selection).selectAll(mine).data(dataArray(data, context), key);
    instances.exit().each(destroyInstance);
    return instances.enter().append(tagName).attr('class', className).each(createInstance).merge(instances).each(render);
  }

  function mine() {
    return Array.from(this.children).filter(belongsToMe);
  }

  function belongsToMe(node) {
    var instance = getInstance(node);
    return instance && instance.owner === component;
  }

  function dataArray(data, context) {
    data = Array.isArray(data) ? data : [data];
    return context ? data.map(function (d) {
      return Object.assign(Object.create(context), d);
    }) : data;
  }

  function createInstance(d, i, nodes) {
    var _this = this;

    setInstance(this, {
      owner: component,
      destroy: function destroy() {
        return _destroy.call(_this, d, i, nodes);
      }
    });
    create.call(this, d, i, nodes);
  }

  function destroyInstance() {
    d3Selection.select(this).selectAll('*').each(destroyDescendant);
    (getInstance(this).destroy() || d3Selection.select(this)).remove();
  }

  function destroyDescendant() {
    var instance = getInstance(this);
    if (instance) {
      instance.destroy();
    }
  }

  component.render = function (_) {
    render = _;
    return component;
  };
  component.create = function (_) {
    create = _;
    return component;
  };
  component.destroy = function (_) {
    _destroy = _;
    return component;
  };
  component.key = function (_) {
    key = _;
    return component;
  };

  return component;
};

exports.component = component;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=d3-component.js.map
